# -*- coding: UTF-8 -*-

# --- [ cl_Editor ] --------------------------------------------------------------
#
# Textausgabefelder werden mit vertikalem und optional
# vertikalem Scrollbar zusammengestellt
# Ein Button zum Löschen des gesamten Fensterinhaltes ist eingebaut.

import tkinter as tk


class Editor(tk.Text):
    def __init__(self, master,
                 row=0,
                 column=0,
                 columnspan=1,
                 rowspan=1,
                 width=40,
                 height=10,
                 xscroll=True,
                 **kw):
        
        tk.Text.__init__(self, master,
                         height=height,
                         width=width,
                         **kw)

        self.grid(column=column, row=row, rowspan=rowspan, columnspan=columnspan, padx=5, pady=5, sticky="nsew")

        # Der Editor erhält einen Scrollbar für die vertikale Richtung.
        yscrollbar = tk.Scrollbar(master)
        yscrollbar.grid(column=column + columnspan, row=row, rowspan=rowspan, pady=5, sticky="nsw")
        yscrollbar.config(command=self.yview)
        self.configure(yscrollcommand=yscrollbar.set)

        # Und ggf. auch einen Scrollbar für die horizontale Richtung.
        # und einen Button zum Löschen der Textausgabe
        if xscroll:
            xscrollbar = tk.Scrollbar(master, orient="horizontal", width=20)
            xscrollbar.grid(column=column, columnspan=columnspan, row=row + rowspan, padx=5, sticky="new")
            xscrollbar.config(command=self.xview)

            self.configure(xscrollcommand=xscrollbar.set, wrap="none")

            self.__btn = tk.Button(master, text="X", height=1, command=self.__delete)
            self.__btn.grid(column=column + columnspan, row=row + rowspan, sticky="nw")

        else:
            self.__btn = tk.Button(master, text="Delete Log", command=self.__delete)
            self.__btn.grid(column=column + columnspan, row=row + rowspan, columnspan=2, sticky="nsew")
            
    def __delete(self):
        self.delete("1.0", 'end')

if __name__ == "__main__":
    top = tk.Tk()
    top.title("Dali-Logger")

    frame1 = tk.LabelFrame(top, padx=5, pady=5, text="Test", height=40, width=40, relief="groove")
    frame1.grid(column=0, row=0, padx=5, pady=5)

    tx1 = Editor(frame1, column=0, row=0, rowspan=4, width=80, height=40)

    tx1.insert('end', 'TExt' + '\n')
    tx1.insert('end', 'TExt' + '\n')

    top.mainloop()
