#!/usr/bin/python
# -*- coding: UTF-8 -*-
import tkinter as Tk

class Entry(Tk.Entry):
    """ Ein Label plus ein Entry-Widget
        nimmt Integer und Hexvariablen an """
    # Ein Callback, das bei "RETURN" aufgerufen wird
    cb_return = None

    def __init__(self, master, label = None, value = 0, column = 0, row = 0, minval = 0, maxval = 255):
        Tk.Label(master, text = label, width = 10, anchor = 'e').grid(column = column, row = row)
        self.__var = Tk.StringVar()
        self.__var.set(str(value))
        self.__minval = minval
        self.__maxval = maxval

        self.__entry  = Tk.Spinbox(
                master,                     # das Parent-Window
                relief = "sunken",
                state = "normal",           #
                from_ = self.__minval,      # Wertebereich von ..
                to = self.__maxval,         # Wertebereich .. bis
                increment = 1,              # Intervall der Werte
                textvariable = self.__var,  # die steuernde Variable
                justify = "left",
                width = 5                   # die Breite des Eingabefeldes
                )

        self.__entry.grid(column = column + 1, row = row,
                          padx = 5, pady = 5, sticky = "nse")

        self.__check = self.__entry.register(self.isOK)
        self.__entry.configure(validate = 'key', validatecommand = (self.__check, '%P'))
        self.__entry.bind('<Key-Return>', self.__return)

    def __return(self, event):
        if Entry.cb_return is not None:
            Entry.cb_return()

    def set(self, value):
        self.__var.set(value)

    def set_state(self, state):
        if state:
            self.__entry.configure(state = 'normal')
        else:
            self.__entry.configure(state = 'disabled')

    def set_callback(self, callback):
        Entry.cb_return = callback

    def set_state(self, state):
        if state:
            self.__entry.config(state = "normal")
        else:
            self.__entry.config(state = "disabled")

    def get(self):
        """ Die Variable kann auch leer sein ! """
        i = self.__var.get()
        if not i:
            i = 0
        try:
            i = int(i)
        except:
            i = int(i, base = 16)

        return i

    def isOK(self, new_value):
        """ Achtung: self.__var.get() liefert den unveränderten Wert zurück ! """
        if not new_value:            # Eine leeren Eingabe akzeptieren
            #self.__entry.bell()
            return True

        if new_value == "-":        # Negative Werte zulassen
            return True

        # Prüfung der Eingabe auf Int / Hex (führendes '0x')
        try:
            if new_value.isdigit():
                i = int(new_value)

            elif new_value.upper().startswith("0X"):
                i = new_value[2:]
                if len(i) == 0:
                    i = self.__minval
                else:
                    i = int(new_value[2:], base = 16)

        except:
            self.__entry.bell()
            return False

        # Prüfen des Wertebereiches
        try:
            if i < self.__minval or i > self.__maxval:
                raise BaseException

        except:         # wenn nicht, dann wird die Eingabe rückgängig gemacht
            self.__entry.bell()
            return False

        return True