﻿#! /usr/bin/python3
# -*- coding: UTF-8 -*-

# 30.09.2015

import tkinter as Tk
from tkinter import messagebox
# import sys
import os
import time
import _thread

from cl_Editor import Editor
from cl_Entry import Entry
from cl_Combo import Combo
from rs232 import RS232
from open_comport import open_comport
from open_comport import get_comport_name

START_BYTE = 0xA5
ERROR_BYTE = 0x5A
WRITE1x = 0xA1
WRITE2x = 0xA2
READBITS = 0x81
NOTHING = "nv"
GATEWAY_ERROR = 5

TERMINATE = (0xA1, 0x00)
INITIALISE = (0xA5, 0x00)
RANDOMISE = (0xA7, 0x00)
COMPARE = (0xA9, 0x00)
WITHDRAW = (0xAB, 0x00)
QUERY_SHORT_ADRESS = (0xBB, 0x00)
SEARCH_ADDR_H = 0xB1
SEARCH_ADDR_M = 0xB3
SEARCH_ADDR_L = 0xB5
PROGRAM_SHORT_ADDR = 0xB7
VERIFY_SHORT_ADDR = 0xB9

OLD_RANDOM_EXPORT = "random_export.cfg"
NEW_SHORT_IMPORT = "short_import.cfg"


class App(object):

    def __init__(self, ser=None):
        self.top = Tk.Tk()
        self.top.title("RS232-DALI Gateway @ {}:".format(get_comport_name()))
        self.top.resizable(width=False, height=False)
        self.top.protocol('WM_DELETE_WINDOW', self.quit)

        self.ser = ser
        self.lastcmd = "empty"
        self.send_twice = False
        self.counter = 0
        self.thread_is_activ = False
        self.rx_byte_flag = 0
        self.rx_byte_received = None
        self.received = False

        self.add_widgets()
        self.add_connections()

        self.top.mainloop()

    def add_widgets(self):
        self.frm1 = Tk.LabelFrame(self.top, text="Dali_Byte1 / Adress:", width=200, height=100)
        self.frm1.grid(column=0, columnspan=2, row=0, padx=5, pady=5, sticky="nsew")

        #                 Bezeichner, Wert, Spalte, Zeile
        radiobuttons = (
                        ("Short Adress:", 1, 1, 0),
                        ("Group Adress:",  2, 1, 1),
                        ("Broadcast",  4, 1, 2),
                        ("Special Cmd Byte1:",  8, 1, 3))

        self.rb1 = Tk.IntVar()
        self.rb1.set(1)

        for r in radiobuttons:
            Tk.Radiobutton(self.frm1, text=r[0], variable=self.rb1, value=r[1], command=self.rb1set).grid(
                column=r[2], row=r[3], padx=5, pady=5, sticky="nsw")

        self.dali_adr  = Entry(self.frm1, label="[ 0 .... 63 ]", value=1, column=2, row=0, maxval=63)
        self.dali_grp  = Entry(self.frm1, label="[ 0 .... 15 ]", value=0, column=2, row=1, maxval=15)
        self.dali_cmd1 = Entry(self.frm1, label="[ 161 . 190 ]", value=161, column=2, row=3)

        # Byte1 in Special-Commands vorbelegen mit "Data Transfer Register (DTR)"
        self.dali_cmd1.set("0xA3")

        # die Eingabefelder freigeben/sperren
        #self.dali_adr.set_state(0)

        self.frm2 = Tk.LabelFrame(self.top, text="Dali_Byte2 / Data", width=200, height=100)
        self.frm2.grid(column=2, columnspan=2, row=0, padx=5, pady=5, sticky="nsew")

        self.rb2 = Tk.IntVar()
        self.rb2.set(1)
        #               Bezeichner, Wert, Spalte, Zeile
        radiobuttons = (
                        ("Direct Arc:", 0, 0, 0),
                        ("Command:",  1, 0, 1))
        i = 0
        for r in radiobuttons:
            Tk.Radiobutton(self.frm2, text=r[0], variable=self.rb2, value=r[1], command=self.rb2set).grid(
                        column=r[2], row=r[3], padx=5, pady=5, sticky="nsw")
            i += 1

        self.dali_arc = Entry(self.frm2, label="[0 ... 255]", value=0,  column=2, row=0, minval=0, maxval=255)
        self.dali_cmd0 = Entry(self.frm2, label="[0 ... 255]", value=0, column=2, row=1, minval=0, maxval=255)

        self.combo = Combo(self.frm2, column=0, row=2, width=10, callback=self.combo_callback)
        Tk.Label(self.frm2, text="Special Cmd Byte2:   ").grid(column=0, row=4, padx=5, pady=5, sticky="nsw")
        self.dali_cmd2 = Entry(self.frm2, label="[0 ... 255]", value=0, column=2, row=4, minval=0, maxval=255)

        self.frm3 = Tk.LabelFrame(self.top, text="Send Cmd/Arc")
        self.frm3.grid(column=0, columnspan=4, row=1, padx=5, pady=5, sticky="nsew")
        #             Buttontext, Spalte, Zeile, command
        Tk.Label(self.frm3, width=40).grid(column=0, columnspan=2, row=0, padx=5, pady=5)
        buttons = (
                    ("Send 1x",             2, 0, self.dali_send),
                    ("Send 2x",             3, 0, self.dali_send2)
                    )
        for b in buttons:
            Tk.Button(self.frm3, text=b[0], command=b[3], width=18).grid(
                    column=b[1], row=b[2], padx=5, pady=5, sticky="nsew")

        self.frm4 = Tk.LabelFrame(self.top, text="Macros")
        self.frm4.grid(column=0, columnspan=4, row=2, padx=5, pady=5, sticky="nsew")
        #             Buttontext, Spalte, Zeile, command
        buttons = (
                    ("Find ShortAdresses",  0, 1, self._find_short_adr),
                    ("Find RandomAdresses", 0, 2, self._find_random_adr),
                    ("Make ShortAdresses",  0, 3, self._make_short_adr),
                    ("Randomise",           0, 4, self._randomise),
                    ("Blink 5x",            3, 1, self._blink5x),
                    ("Query Long",          1, 1, self._query_long),
                    ("Query Short",         1, 2, self._query_short),
                    ("Query Groups",        1, 3, self._query_groups),
                    ("Query DTR",           1, 4, self._query_dtr),
                    ("Query Light Level",   2, 4, self._query_level),
                    ("Recall MAX Level",    2, 1, self._recall_max_level),
                    ("Recall MIN Level",    2, 2, self._recall_min_level),
                    ("Dimm UP",             3, 1, self._dimm_up),
                    ("Dimm DOWN",           3, 2, self._dimm_down),
                    ("Scene Levels",        3, 4, self._query_scene_levels)
                    # ("< Exit >",            3, 4, self.quit)
                    )
        for b in buttons:
            Tk.Button(self.frm4, text=b[0], command=b[3], width=18).grid(
                    column=b[1], row=b[2], padx=5, pady=5, sticky="nsew")

        # die Fensterhöhe reduzieren, wenn auf einem Netbook gearbeitet wird
        if self.top.winfo_screenheight() > 600:
            editor_height = 23
        else:
            editor_height = 5
        self.tx = Editor(self.top, row=6, height=editor_height, colspan=4)

        Tk.Button(self.top, text="< Exit >", width=18, command=self.quit).grid(
            column=2, columnspan=2, row=8, padx=5, pady=5, sticky="nse")

    def add_connections(self):
        self.rb1set()
        self.rb2set()

        # Ein Return in einem Entry führt ein "Senden" aus
        # gilt für alle Instanzen von Entry
        self.dali_adr.set_callback(self.dali_send)

        # Hintergrundthread zum Empfangen von Daten via Serieller Schnittstelle
        self.rs232 = RS232(ser=self.ser, callback=self.rs232_callback)
        self.rs232.start()

    def rb1set(self):
        """
        Wird aufgerufen, wenn der Radiobutton1 (Auswahl adr, group, broadcast, sepecialcmd) geändert wird
        """
        if self.rb1.get() == 1:
            self.dali_adr.set_state(1)
            self.dali_grp.set_state(0)
            self.dali_cmd1.set_state(0)
            self.dali_cmd2.set_state(0)
        elif self.rb1.get() == 2:
            self.dali_adr.set_state(0)
            self.dali_grp.set_state(1)
            self.dali_cmd1.set_state(0)
            self.dali_cmd2.set_state(0)
        elif self.rb1.get() == 4:
            self.dali_adr.set_state(0)
            self.dali_grp.set_state(0)
            self.dali_cmd1.set_state(0)
            self.dali_cmd2.set_state(0)
        elif self.rb1.get() == 8:
            self.dali_adr.set_state(0)
            self.dali_grp.set_state(0)
            self.dali_cmd1.set_state(1)
            self.dali_cmd2.set_state(1)

    def rb2set(self):
        """
        Wird aufgerufen, wenn der Radiobutton1 ( Auswahl arc/cmd ) geändert wird
        """
        if self.rb2.get() == 0:
            self.dali_arc.set_state(1)
            self.dali_cmd0.set_state(0)
        elif self.rb2.get() == 1:
            self.dali_arc.set_state(0)
            self.dali_cmd0.set_state(1)

    def combo_callback(self, data):
        """
        Bei Änderungen in der Combobox für die Commands wird der neue Wert in das Entry-Feld für das Command eingetragen
        """
        self.dali_cmd0.set(data)

    def check_adr_selection(self, txt="command"):
        """
        Wird ausgeführt vor Commands, die für ShortAdr, GroupAdr oder Broadcast möglich sind.
        Wenn aber GroupAdr oder Broadcast ausgewählt ist, dann wird nachgefragt, ob das Command
        ausgeführt werden soll.
        :param txt: Fenstertitel und gleichzeitig Hinweistext
         :return: -1 bei Fehler, sonst die fertig formatierte Adresse als DALI-Byte1
        """

        selection = self.rb1.get()
        if selection == 1:
            adr = self.dali_adr.get()
            self.dali_send_data(((adr << 1)+1, 0x99))
            if self.rx_byte_flag == GATEWAY_ERROR:
                return -1
            elif self.rx_byte_flag:
                messagebox.showerror("Check Adr Selection", "ShortAdr '{:02}' not found !".format(adr))
                return -1
            else:
                return (adr << 1) + 1

        elif selection == 2:
            t = "{} for Group_Adr {:02} ?".format(txt, self.dali_grp.get())
            adr = ((self.dali_grp.get() << 1) + 0x81)

        elif selection == 4:
            t = "{} as Broadcast ?".format(txt)
            adr = 0xFF
        else:
            messagebox.showinfo(txt, "Invalid with Special Command !")
            return -1

        if messagebox.askyesno(txt, t):
            return adr
        else:
            return -1

    def check_Gateway_Error(self):
        if self.rx_byte_flag == GATEWAY_ERROR:
            self.tx.insert('end', 'GatewayError\n')
            self.tx.see('end')
            return True
        else:
            return False

    def check_if_shortadr_is_selected(self):
        """
        Wird ausgeführt vor Commands, die nur für ShortAdressen möglich sind.
        Prüft, ob der Radiobutton ShortAdr ausgewählt,
        wenn ja, dann wird geprüft, ob die Adresse auf ein Command 0x99 antwortet
        :return: Shortadr oder -1
        """
        if self.rb1.get() > 1:
            messagebox.showerror("Error", "Select a ShortAdress !")
            return -1
        else:

            adr = self.dali_adr.get()
            self.dali_send_data(((adr << 1) + 1, 0x99))
            if self.rx_byte_flag == 5:
                # messagebox.showerror("Check ShortAdr", "Gateway not found !")
                return -1
            elif self.rx_byte_flag:
                messagebox.showerror("Check ShortAdr", "ShortAdr '{:02}' not found !".format(adr))
                return -1
            else:
                return adr

    def thread_query_scene_levels(self, adr):

        self.thread_is_activ = True
        cmd1 = (adr << 1) + 1
        self.tx.insert('end', "\n")
        for i in range(16):
            scene_level = self.dali_send_data((cmd1, 0xB0 + i))
            if self.rx_byte_flag:
                self.tx.insert('end', "ShortAdr: {:02} not found !\n".format(adr))
            else:
                self.tx.insert('end', "Query ShortAdr: {:02} Scene: {:02} Level: {:003}\n".format(adr, i, scene_level))

            self.tx.see('end')
            if self.rx_byte_flag:
                break
        self.thread_is_activ = False

    def thread_find_short_adr(self, unused):
        self.thread_is_activ = True
        cnt = 0
        self.tx.insert('end', "Looking for ShortAdresses:\n")
        self.tx.see('end')
        for adr in range(64):
            shortadr = (adr << 1) + 1
            adrh = self.dali_send_data((shortadr, 0xC2))
            if self.check_Gateway_Error():
                return
            elif self.rx_byte_flag:
                self.tx.insert('end', "{:02}, ".format(adr))
                # Zeilenumbruch nach 8 Adressen
                if not (adr + 1) % 8:
                    self.tx.insert('end', "\n")
                    self.tx.see('end')
                continue
            else:
                cnt += 1
                adrm = self.dali_send_data((shortadr, 0xC3))
                adrl = self.dali_send_data((shortadr, 0xC4))

            self.tx.insert('end', "\nShortAdr found: {:02} RandomAdr: {:03} {:03} {:03}\n".format(
                adr, adrh, adrm, adrl))
            self.tx.see('end')

        self.tx.insert('end', "\n{} Adresses found !\n".format(cnt))
        self.tx.see('end')
        self.thread_is_activ = False

    def search_randomadr(self, dezadr):
        """
        Nimmt eine Dezimale Adresse entgegen, versendet sie und testet, ob diese Adresse mit der Random-Adresse
        übereinstimmt.
        :param dezadr:
        :return: 0 = Adresse kleiner als RandomAdr/ 255 = Adresse größer/gleich RandomAdr
        """
        adr = self.adr_2_bytes(dezadr)

        self.dali_send_data((SEARCH_ADDR_H, adr[0]))
        self.dali_send_data((SEARCH_ADDR_M, adr[1]))
        self.dali_send_data((SEARCH_ADDR_L, adr[2]))

        i = self.dali_send_data(COMPARE)
        if i == NOTHING:
            i = 0
        return i

    def dali_send2(self):
        self.send_twice = True
        self.dali_send()

    def dali_send(self):
        """
        Wird ausgeführt, wenn der Button 'Senden' gedrückt wird
        oder innerhalb eines Entryfelder ein 'Enter' eingegeben wird
        """
        if self.thread_is_activ:
            return

        # Radiobutton "direct arc / cmd"
        j = self.rb2.get()
        if j:
            byte2 = self.dali_cmd0.get()
            # Reset - wirklich ??
            if byte2 == 0x20:
                if messagebox.askyesno("Warnung", "RESET abbrechen ?"):
                    self.send_twice = False
                    return
        else:
            byte2 = self.dali_arc.get()
            if self.rb1.get() < 8:
                self.send_twice = False

        # Radiobutton "Short Adr, Group Adr, Broadcast, Special Command"
        i = self.rb1.get()
        byte1 = 0
        # Short Adress
        if i == 1:
            byte1 += (self.dali_adr.get() << 1) + j
            t = "ShortAdr: {:02}".format(self.dali_adr.get())

        # Group Adress
        elif i == 2:
            byte1 += (self.dali_grp.get() << 1) + j
            byte1 += 0x80
            t = "GroupAdr: {:02}".format(self.dali_grp.get())

        # Broadcast
        elif i == 4:
            byte1 = 0xFE + j
            t = "Broadcast:  "

        # Special Command
        elif i == 8:
            t = "SpecialCmd: "
            byte1 = self.dali_cmd1.get()
            if byte1 < 161 or byte1 > 190:
                self.tx.bell()
                self.tx.insert('end', "Error in special byte1 -> {:02x} / {}\n".format(byte1, byte1))
                return
            byte2 = self.dali_cmd2.get()

        self.lastcmd = "{} -> {:02X} {:02X} - {:08b} {:08b}".format(t, byte1, byte2, byte1, byte2)

        # Prüfen, ob eine Query an eine Gruppe oder als Broadcast versendet werden soll
        if (byte1 & 0xE1 == 0x81) or (byte1 & 0xE1 == 0xE1):
            if byte2 >= 0x90:
                messagebox.showerror("CommandError", "QUERY in Group or Broadcast -> (0x{:02X}, 0x{:02X}) !".
                                     format(byte1, byte2))
                return

        self.dali_send_data((byte1, byte2), self.send_twice)
        self.tx.insert('end', "{} {} \n".format(self.lastcmd, self.__format_status()))

        # Checkbutton für 2x senden ist gesetzt
        if self.send_twice:
            self.tx.insert('end', "------------- > send 2x\n")
            self.send_twice = False
        self.tx.see('end')

    def dali_send_data(self, cmd, send_twice=False):
        """
        Führt ein Command aus und liefert die Rückmeldung zurück
        :param cmd: Tuple aus 2 Bytes
        :return: Das empfangene Byte
        """
        self.received_byte = None
        self.received = False

        if send_twice:
            self.ser.write(bytes((START_BYTE, cmd[0], cmd[1], WRITE2x)))
        else:
            self.ser.write(bytes((START_BYTE, cmd[0], cmd[1], WRITE1x)))

        # warten, bis die Antwort eingegangen ist
        # das dauert - wenn der Slave nicht antwortet: 15.83ms + 9.17ms = 25ms
        #              wenn der Slave antwortet 15.83 + (2.92 - 9.17ms) + 9.17ms
        # bei send_twice = True jeweils die doppelte Zeit
        i = 0
        while self.received is False:
            time.sleep(0.005)
            i += 1
            if i > 20:
                messagebox.showerror("DALI-Read", "No Answer from Gateway !")
                self.rx_byte_flag = GATEWAY_ERROR
                return 0

        # self.rx_byte_flag zeigt den Erfolg an
        # wenn == 0, dann wurde ein Wert empfangen
        if self.rx_byte_flag == 0:
            return self.rx_byte_received
        else:
            return NOTHING

    def rs232_callback(self, data):
        """
        :param data: die Daten, die von der Seriellen Schnittstelle empfangen wurden, als Liste
        :return:
        rs232_callback für den Empfang der Daten über die serielle Schnittstelle
        Die empfangenen Daten werden als List übergeben.
        Das Flag received signalisiert, dass Daten  empfangen wurden
        Das Byte1 wird an eine (leere) Liste angehängt
        """

        # Das Flag zeigt die Rückmeldung des Gateways zu Rückmeldung des DALI-Slaves an
        # 0 = Daten empfangen
        # 1 = keine Rückmeldung
        # 1/2 Fehler im Bit / Byte Timing
        self.rx_byte_flag = data[0]
        # Das Datenbyte wird an die Liste rx_bytes_list angefügt
        self.rx_byte_received = data[1]
        # Das Flag wird benutzt, um den Eingang der Antwort zu signalisieren
        # Daraufhin beendet dali_write_bytes() seine Warteschleife
        self.received = True

    def _blink5x(self, cnt=5):
        """
        Schaltet die gewählten Lampen  x-mal ein / aus
        Es kann nicht erkannt werden, ob sich Geräte angesprochen fühlen:
        Möglicherweise bleibt der Button für die Blinkzeit gedrückt, aber es zeigt sich keinerlei Aktivität
        """
        if self.thread_is_activ:
            return

        adr = self.check_adr_selection("Blink5x")
        if adr < 0:
            return

        for i in range(cnt):
            self.dali_send_data((adr, 0x05))
            if self.check_Gateway_Error():
                return
            time.sleep(.3)
            self.dali_send_data((adr, 0x06))
            time.sleep(.3)

    def _query_dtr(self):
        """
        Funktioniert nur für eine Shortadresse
        Fragt das DTR für die aktuell gewählte ShortAdr ab und gibt der Wert im Editorfenster aus
        """
        if self.thread_is_activ:
            return

        adr = self.check_if_shortadr_is_selected()
        if adr < 0:
            return
        cmd = (adr << 1) + 1
        self.tx.insert('end', "Query ShortAdr: {:02} DTR: {}\n".format(adr, self.dali_send_data((cmd, 0x98))))
        self.tx.see('end')

    def _recall_max_level(self):
        """
        Setzt die gewählte Adressen auf Maximales Level
        """
        if self.thread_is_activ:
            return
        cmd = self.check_adr_selection("Recall MAX Level")
        if cmd < 0:
            return

        self.dali_send_data((cmd, 0x05))

    def _recall_min_level(self):
        """
        Setzt die gewählte Adressen auf Minimales Level
        """
        if self.thread_is_activ:
            return

        cmd = self.check_adr_selection("Recall MIN Level")
        if cmd < 0:
            return
        self.dali_send_data((cmd, 0x06))

    def _dimm_up(self):
        """
        Dimmt die gewählten Adressen hin zu helleren Werten
        siehe
        """
        if self.thread_is_activ:
            return
        cmd = self.check_adr_selection("Dimm Up")
        if cmd < 0:
            return

        self.dali_send_data((cmd, 0x01))

    def _dimm_down(self):
        """
        Dimmt die gewählten Adressen hin zu dunkleren Werten
        """
        if self.thread_is_activ:
            return

        cmd = self.check_adr_selection("Dimm Down")
        if cmd < 0:
            return

        self.dali_send_data((cmd, 0x02))

    def _query_level(self):
        """
        Fragt das Lampenlevel für die aktuell gewählte ShortAdr ab und gibt der Wert im Editorfenster aus
        """
        if self.thread_is_activ:
            return

        adr = self.check_if_shortadr_is_selected()
        if adr < 0:
            return
        cmd = (adr << 1) + 1

        self.tx.insert('end', "Query ShortAdr: {:02} Level: {}\n".format(adr, self.dali_send_data((cmd, 0xA0))))

    def _query_groups(self):
        """
        Fragt das Lampenlevel für die aktuell gewählte ShortAdr ab und gibt der Wert im Editorfenster aus
        """
        if self.thread_is_activ:
            return

        adr = self.check_if_shortadr_is_selected()
        if adr < 0:
            return
        
        cmd = (adr << 1) + 1

        group_low = self.dali_send_data((cmd, 0xC0))
        group_high = self.dali_send_data((cmd, 0xC1))

        self.tx.insert('end', "Query ShortAdr: {:02} Groups: {:08b} {:08b}\n".format(adr, group_high, group_low))
        self.tx.see('end')

    def _query_scene_levels(self):
        """
        Fragt das Lampenlevel für die aktuell gewählte ShortAdr ab und gibt die Werte im Editorfenster aus
        """
        if self.thread_is_activ:
            return

        adr = self.check_if_shortadr_is_selected()
        if adr < 0:
            return

        _thread.start_new_thread(self.thread_query_scene_levels, (adr,))

    def _find_short_adr(self):
        """
        Durchsucht den Adressraum von 0 - 63 nach antwortenden Slaves.
        Die Short und die RandomAdr werden im Editorfenster ausgegeben.
        """
        if self.thread_is_activ:
            return
        _thread.start_new_thread(self.thread_find_short_adr, (None,))

    def _query_long(self):
        """
        Fragt alle Parameter der aktuell gewählten ShortAdr ab und gibt
        eine Liste im Editorfenster aus.
        Damit die Anzeige laufend aktualisiert wird, muss ein Thread gestarted werden
        """
        if self.thread_is_activ:
            return
        
        adr = self.check_if_shortadr_is_selected()
        if adr < 0:
            return
        cmd = (adr << 1) + 1
        print(self.dali_send_data((cmd, 0x99)))

        _thread.start_new_thread(self.thread_query_long, (None,))

    def thread_query_long(self, para):
        self.thread_is_activ = True
        shortadr = self.dali_adr.get()
        adr = (shortadr << 1) + 1
        self.tx.insert('end', "\nQUERY SHORT ADRESS: ------ {:02}\n".format(shortadr))

        t = "Query Status             : {}\n".format(self.dali_send_data((adr, 0x90)))
        if self.rx_byte_flag:
            self.tx.insert('end', "ShortAdr: {:02} not found !\n".format(shortadr))
            self.tx.see('end')
            return

        self.tx.insert('end', t)
        self.tx.insert('end', "Query Ballast            : {}\n".format(self.dali_send_data((adr, 0x91))))
        self.tx.see('end')
        self.tx.insert('end', "Query Lamp Failure       : {}\n".format(self.dali_send_data((adr, 0x92))))
        self.tx.see('end')
        self.tx.insert('end', "Query Lamp Power on      : {}\n".format(self.dali_send_data((adr, 0x93))))
        self.tx.see('end')
        self.tx.insert('end', "Query Lamp Limit Err     : {}\n".format(self.dali_send_data((adr, 0x94))))
        self.tx.see('end')
        self.tx.insert('end', "Query Reset State        : {}\n".format(self.dali_send_data((adr, 0x95))))
        self.tx.see('end')
        self.tx.insert('end', "Query Missing ShortAdr   : {}\n".format(self.dali_send_data((adr, 0x96))))
        self.tx.see('end')
        self.tx.insert('end', "Query Version Nr         : {}\n".format(self.dali_send_data((adr, 0x97))))
        self.tx.see('end')
        self.tx.insert('end', "Query Content DTR        : {}\n".format(self.dali_send_data((adr, 0x98))))
        self.tx.see('end')
        self.tx.insert('end', "Query Device Typ         : {}\n".format(self.dali_send_data((adr, 0x99))))
        self.tx.see('end')
        self.tx.insert('end', "Query Phys. Minimum Lvl  : {}\n".format(self.dali_send_data((adr, 0x9A))))
        self.tx.see('end')
        self.tx.insert('end', "Query Power Failure      : {}\n".format(self.dali_send_data((adr, 0x9B))))
        self.tx.see('end')
        self.tx.insert('end', "Query Actual Level       : {}\n".format(self.dali_send_data((adr, 0xA0))))
        self.tx.see('end')
        self.tx.insert('end', "Query Max Level          : {}\n".format(self.dali_send_data((adr, 0xA1))))
        self.tx.see('end')
        self.tx.insert('end', "Query Min Level          : {}\n".format(self.dali_send_data((adr, 0xA2))))
        self.tx.see('end')
        self.tx.insert('end', "Query Power On Level     : {}\n".format(self.dali_send_data((adr, 0xA3))))
        self.tx.see('end')
        self.tx.insert('end', "Query System Failure Lvl : {}\n".format(self.dali_send_data((adr, 0xA4))))
        self.tx.see('end')
        self.tx.insert('end', "Query Fade Time/Fade Rate: 0x{:02X}\n".format(self.dali_send_data((adr, 0xA5))))
        self.tx.see('end')

        for i in range(16):
            scene_level = self.dali_send_data((adr, 0xB0 + i))
            self.tx.insert('end', "Query Scene: {:02}  -  Level: {:003}\n".format(i, scene_level))
            self.tx.see('end')

        self.tx.insert('end', "Query Groups 0- 7        : 0b{:08b}\n".format(self.dali_send_data((adr, 0xC0))))
        self.tx.see('end')
        self.tx.insert('end', "Query Groups 8-15        : 0b{:08b}\n".format(self.dali_send_data((adr, 0xC1))))
        self.tx.see('end')
        self.tx.insert('end', "Query Random Adr High    : {}\n".format(self.dali_send_data((adr, 0xC2))))
        self.tx.see('end')
        self.tx.insert('end', "Query Random Adr Mid     : {}\n".format(self.dali_send_data((adr, 0xC3))))
        self.tx.see('end')
        self.tx.insert('end', "Query Random Adr Low     : {}\n".format(self.dali_send_data((adr, 0xC4))))
        self.tx.see('end')
        self.thread_is_activ = False

    def _query_short(self):
        """
        Fragt ausgewählte Parameter der aktuell gewählten ShortAdr ab und gibt
        eine Liste im Editorfenster aus.
        Damit die Anzeige laufend aktualisiert wird, muss Thread gestarted werden
        """
        if self.thread_is_activ:
            return

        adr = self.check_if_shortadr_is_selected()
        if adr < 0:
            return

        _thread.start_new_thread(self.thread_query_short, (None,))

    def thread_query_short(self, unused):

        self.thread_is_activ = True
        shortadr = self.dali_adr.get()
        adr = (shortadr << 1) + 1
        self.tx.insert('end', "\nQUERY SHORT ADRESS: ------ {:02}\n".format(shortadr))

        t = "Query Content DTR        : {}\n".format(self.dali_send_data((adr, 0x98)))
        if self.rx_byte_flag:
            self.tx.insert('end', "ShortAdr: {:02} not found !\n".format(shortadr))
            self.tx.see('end')
            return

        self.tx.insert('end', t)
        self.tx.see('end')
        self.tx.insert('end', "Query Device Typ         : {}\n".format(self.dali_send_data((adr, 0x99))))
        self.tx.see('end')
        self.tx.insert('end', "Query Phys. Minimum Lvl  : {}\n".format(self.dali_send_data((adr, 0x9A))))
        self.tx.see('end')
        self.tx.insert('end', "Query Actual Level       : {}\n".format(self.dali_send_data((adr, 0xA0))))
        self.tx.see('end')
        self.tx.insert('end', "Query Max Level          : {}\n".format(self.dali_send_data((adr, 0xA1))))
        self.tx.see('end')
        self.tx.insert('end', "Query Min Level          : {}\n".format(self.dali_send_data((adr, 0xA2))))
        self.tx.see('end')
        self.tx.insert('end', "Query Power On Level     : {}\n".format(self.dali_send_data((adr, 0xA3))))
        self.tx.see('end')
        self.tx.insert('end', "Query Fade Time/Fade Rate: 0x{:02X}\n".format(self.dali_send_data((adr, 0xA5))))
        self.tx.see('end')
        self.tx.insert('end', "Query Groups 0- 7        : 0b{:08b}\n".format(self.dali_send_data((adr, 0xC0))))
        self.tx.see('end')
        self.tx.insert('end', "Query Groups 8-15        : 0b{:08b}\n".format(self.dali_send_data((adr, 0xC1))))
        self.tx.see('end')
        self.thread_is_activ = False

    def _randomise(self):
        """
        Hier werden an allen angeschlossenen Geräten neu 3Byte RandomAdressen generiert.
        :return:
        """
        if messagebox.askyesno("Randomise", "Make New RandomAdresses ?"):
            if not messagebox.askyesno("Randomise", "Abort Randomising ?"):
                # Initialise 2x innerhalb 100ms
                self.dali_send_data(INITIALISE)
                if self.check_Gateway_Error():
                    return
                self.dali_send_data(INITIALISE)
                # Randomise 2x innerhalb 100ms
                self.dali_send_data(RANDOMISE, send_twice=True)
                # Terminate
                self.dali_send_data(TERMINATE)

    def _find_random_adr(self):
        """
        Durchsucht den Bus nach den RandomAdressen
        zeigt alle Fundstellen an und kann die Adressen in eine Datei schreiben.
        :return:
        """
        if self.thread_is_activ:
            return

        if not messagebox.askyesno("Find Random Adresses", "Search for Random Adresses ?"):
            return
        if messagebox.askyesno("Find Random Adresses", "Write Data to file {} ?".format(OLD_RANDOM_EXPORT)):
            _thread.start_new_thread(self.thread_commissioning, (OLD_RANDOM_EXPORT, None))
        else:
            _thread.start_new_thread(self.thread_commissioning, (None, None))

    def _make_short_adr(self):
        if not messagebox.askyesno("Make ShortAdresses", "Generate New ShortAdresses ???"):
            return

        if os.path.isfile(NEW_SHORT_IMPORT):
            if messagebox.askyesno("Make ShortAdresses",
                                   "Read ShortAdresses from file: '{}' ???".format(NEW_SHORT_IMPORT)):
                if not self.check_newadr_file():
                    return
                _thread.start_new_thread(self.thread_commissioning, (None, NEW_SHORT_IMPORT))
                return

        if messagebox.askyesno("Make ShortAdresses", "Count ShortAdresses UP from 0 ???"):
            _thread.start_new_thread(self.thread_commissioning, (None, 1))

        else:
            if messagebox.askyesno("Make ShortAdresses", "Count ShortAdresses DOWN from 63 ???"):
                _thread.start_new_thread(self.thread_commissioning, (None, 63))
            else:
                messagebox.showerror("Make ShortAdresses", "No More Options !")
                return

    def thread_commissioning(self, oldadrfile, newadrfile):
        if type(oldadrfile) is str:
            oldadrfile = open(oldadrfile, 'w+')

        make_new_adr = False
        direction = -1
        if newadrfile is not None:
            make_new_adr = True
            if type(newadrfile) is str:  # ein Dateiname wurde übergeben, aus dem die Adressen gelesen werden sollen
                try:
                    newadrfile = open(newadrfile)
                except:
                    messagebox.showerror("Error", "File '{}' not found !".format(newadrfile))
                    return
            else:
                if newadrfile == 63:        # die höchste Adresse als Startadresse gewählt
                    nextshortadr = 63
                    direction = 0           # downcounting
                else:                       # die niedrigste Adresse als Startadresse
                    nextshortadr = 0
                    direction = 1           # upcounting
                newadrfile = None

        self.thread_is_activ = True

        # Initialise 2x innerhalb 100ms
        self.dali_send_data(INITIALISE, send_twice=True)

        if self.check_Gateway_Error():
            TRUE = False
        else:
            TRUE = True

        while TRUE:
            longadr = self._ballast_search()
            if longadr is None:
                self.tx.insert('end', ' no (more) RandomAdr found\n')
                self.tx.see('end')

                # Terminate
                self.dali_send_data(TERMINATE)
                break

            # die ShortAdresse einlesen und im Editorfenster ausgeben
            shortadr = (self.dali_send_data(QUERY_SHORT_ADRESS)) // 2
            self.tx.insert('end', " RandomAdr: {:03},{:03},{:03} ShortAdr: {:03}\n".format(
                longadr[0], longadr[1], longadr[2], shortadr))
            self.tx.see('end')

            # Ergebnis in Datei schreiben
            if oldadrfile is not None:
                shortadr = (self.dali_send_data(QUERY_SHORT_ADRESS)) >> 1
                oldadrfile.write("{:03},{:03},{:03},{:03}\n".format(longadr[0], longadr[1], longadr[2], shortadr))

            elif make_new_adr is True:
                if newadrfile is not None:
                    d = newadrfile.readline().strip().split(",")
                    oldlongadr = (int(d[0]), int(d[1]), int(d[2]))
                    newshortadr = int(d[4])
                    if oldlongadr == longadr:
                        if newshortadr >= 0:
                            self.dali_send_data((PROGRAM_SHORT_ADDR, ((newshortadr << 1) + 1)))
                            success = self.dali_send_data((VERIFY_SHORT_ADDR, (0x01 + newshortadr << 1)))
                            if success:
                                self.tx.insert('end', "New Shortadr: {:02}\n".format(int(d[4])))
                                self.tx.see('end')
                        else:
                            self.tx.insert('end', 'Shortadr {} skipped\n'.format(shortadr))
                            self.tx.see('end')
                    else:
                        self.tx.insert('end', "Missmatch: {} <-> {}\n".format(oldlongadr, longadr))
                        self.tx.see('end')

                else:
                    self.dali_send_data((PROGRAM_SHORT_ADDR, ((nextshortadr << 1) + 1)))
                    success = self.dali_send_data((VERIFY_SHORT_ADDR, (0x01 + (nextshortadr << 1))))
                    if success:
                        self.tx.insert('end', "New Shortadr: {:02}\n".format(nextshortadr))
                        self.tx.see('end')

                    if direction:
                        nextshortadr += 1
                    else:
                        nextshortadr -= 1

            self.dali_send_data(WITHDRAW)

        if newadrfile is not None:
            newadrfile.close()
        if oldadrfile is not None:
            oldadrfile.close()
        self.thread_is_activ = False

    def _ballast_search(self):
        """
        Sucht den Adressraum nach der niedrigsten RandomAdr ab
        :return: Gibt die gefundene Adresse zurück, oder 'None'
        """
        upperlimit = 256 * 256 * 256
        lowerlimit = 0
        i = 0
        while True:
            searchadr = lowerlimit + (upperlimit - lowerlimit) // 2
            # print(i, lowerlimit, searchadr, upperlimit)
            self.tx.insert('end', '-')
            self.tx.see('end')

            i += 1
            # Wenn die Suchadresse > oder gleich der Randomadr ist
            # dann ist die Randomadr = oder kleiner als die Suchadr
            if self.search_randomadr(searchadr):
                upperlimit = searchadr
            # wenn die Suchadresse kleiner als die Randomadr ist
            # dann ist die gesuchte Randomadr mindestens um 1 größer
            else:
                lowerlimit = searchadr + 1

            if (upperlimit - lowerlimit) == 0:
                # keine Adresse mehr gefunden
                if upperlimit >= 2**24 - 1:
                    return None

                break
        return self.adr_2_bytes(upperlimit)

    def check_newadr_file(self):
        """
        Prüft das Inputfile daraufhin, ob jede Zeile genau 5 durch Komma getrennte Ziffern enthält
        :return: True if True
        """
        try:
            f = open(NEW_SHORT_IMPORT)
        except:
            messagebox.showerror("Error", "File {} not found".format(NEW_SHORT_IMPORT))
            return False

        cnt = 0
        result = True
        while result is True:
            cnt += 1
            row = f.readline().strip().split(',')

            if len(row) == 1 and row[0] == '':
                result = True
                break

            if not len(row) == 5:
                messagebox.showerror("FileCheck: -> {}".format(NEW_SHORT_IMPORT),
                                     "FieldCntError in row {} -> {}".format(cnt, row))
                result = False
                break

            for r in row:
                try:
                    i = int(r)
                except:
                    messagebox.showerror("FileCheck: -> {}".format(NEW_SHORT_IMPORT), "TypeError in row {} -> {}".format(row, r))
                    result = False
                    break
        f.close()
        return result

    def adr_2_bytes(self, dezadr):
        """
        Formt eine Dezimale Adresse in 3 Bytes um
        :param dezadr:
        :return Adresse als Tuple aus drei Byte
        """
        adrh = dezadr // 65536
        # dezadr = dezadr - (adrh * 65536)
        dezadr -= adrh * 65536
        adrm = dezadr // 256
        adrl = dezadr - adrm * 256

        return adrh, adrm, adrl

    def __format_status(self):
        if not self.rx_byte_flag:
            t = self.rx_byte_received
        elif self.rx_byte_flag == 1:
            t = NOTHING
        elif self.rx_byte_flag == 2:
            t = "BitTimingError"
        elif self.rx_byte_flag == 3:
            t = "ByteTimingError"
        elif self.rx_byte_flag == GATEWAY_ERROR:
            t = "GatewayError"
        else:
            t = "Error [{}]".format(self.rx_byte_flag)
        return t

    def quit(self):
        print("App.quit")
        self.rs232.quit()
        self.top.destroy()

if __name__ == "__main__":
    # die Suche nach Seriellen Porst kann mit start .. ende beschränkt werden
    # wird nur ein Port gefunden, dann wird der geöffnet
    # oder es wird der zu öffnende Ports mit comport = "comX" angegeben
    # s = open_comport(comport = None, start = 1, end = 13)

    App(open_comport())

# --- [ eof ] ----------------------------------------------------------------------------------------------------------
