#!/usr/bin/python
# -*- coding: UTF-8 -*-

import tkinter as tk
import time
from cl_Editor import Editor
from open_comport import open_comport
from open_comport import get_comport_name
from rs232 import RS232
import commandnames as cmd


# def get_cmd_name(command):
#     command = "{:03}".format(command)
#     for c in cmd.commands:
#         if c.startswith(command):
#             return c
#     return "Command not found: {}".format(command)

commands ={}


def init_commands():
    global commands
    for c in cmd.commands:
        # die ersten drei Zeichen enthalten den key
        key = int(c[:3])
        commands.setdefault(key, c)


def get_cmd_name(command):
    return commands.get(command, "Command not found")


def dali_decode(args):
    if args[3] > 1:
        msg=("Bit_Time_OVL", "Byte_Time_OVL", "Bit_Time_Error")
        try:
            txt = msg[args[3]- 2]
        except:
            txt = ""

        return "Error: [3] {:3}-{:3}-{:3}-{:3} {}\n".format(args[0], args[1], args[2], args[3], txt)

    # zuerst die Adresse destillieren
    # group adress oder broadcast
    if args[0] & 128:
        # broadcast
        if args[0] & 0xFE == 0xFE:
            adrstring = "Broadcast   "
        # group adress
        else:
            adrstring = "GroupAdr: {:02}".format((args[0] & 0x1E) > 1)
    # short adress
    else:
        adrstring = "ShortAdr: {:02}".format((args[0] & 0x7E) > 1)

    # command
    if args[0] & 0x01:
        cmdstring = get_cmd_name(args[1])
    # arc level
    else:
        cmdstring = "direct arc level: {:3}".format(args[1])

    # ggf. folgt die Antwort auf ein command, dann ist args[3] == 0
    # bei args[3] == 1 wurde keine Antwort empfangen
    if args[3] == 0:
        # hier steht die Antwort
        backframe = ", Answer: {}".format(args[2])
    else:
        backframe = ""

    return "{}, CMD: {}{}\n".format(adrstring, cmdstring, backframe)


class App(object):
    def __init__(self, serport):
        self.ser = serport

        self.top = tk.Tk()
        self.top.title("RS232-DALI-Logger @ {}:".format(get_comport_name()))
        self.top.resizable(width=False, height=False)
        self.top.protocol('WM_DELETE_WINDOW', self.quit)

        self.starttime = time.time()
        # Thread, der an der Seriellen Schnittstelle lauscht und empfangene Daten per callback
        # zurückliefert
        self.rx = RS232(ser=self.ser, callback=self.callback)
        self.rx.start()

        frame = tk.LabelFrame(self.top, padx=5, pady=5, text="Logging DALI-Bus", relief="groove")
        frame.grid(column=0, columnspan=2, row=0, padx=5, pady=5, sticky="nsew")

        self.chkvar1 = tk.IntVar()
        # self.chkvar.set(1)
        tk.Checkbutton(frame, variable=self.chkvar1, width=25, text="RAW output", anchor="w").grid(
                column=0, row=0, padx=5, pady=7, sticky="nsw")
        self.chkvar2 = tk.IntVar()

        self.chkvar2.set(1)
        tk.Checkbutton(frame, variable=self.chkvar2, width=25, text="show Frametime", anchor="w").grid(
                column=1, row=0, padx=5, pady=7, sticky="nsw")

        # Die Fensterhöhe reduzieren, wenn auf einem Netbook mit 600Pixel Screenheight gearbeitet wird
        if self.top.winfo_screenheight() > 600:
            editor_height = 43
        else:
            editor_height = 25

        self.tx = Editor(self.top, column=0, columnspan=2,  row=1, rowspan=1, width=73, height=editor_height)

        tk.Button(self.top, text="< Exit >", command=self.quit, width=18).grid(
                    column=1, row=3, padx=5, pady=5, sticky="nse")

        # Das Tuple mit den Commands in ein Dictionary umkopieren
        init_commands()

        self.top.mainloop()

        self.rx.quit()
        time.sleep(.1)      # damit der Thread beendet wird, bevor die Serielle Schnittstelle geschlossen wird
        self.ser.close()
        print("ser closed")

    def callback(self, args):
        timestamp = "{:07.2f} - ".format(time.time() - self.starttime)

        if self.chkvar1.get():
            txt = "Dez: {:03}-{:03}-{:03}-{:03} --- Hex: 0x{:02X}-0x{:02X}-0x{:02X}-0x{:02X}\n".format(
                args[0], args[1], args[2], args[3], args[0], args[1], args[2], args[3])
        else:
            txt = dali_decode(args)

        if self.chkvar2.get():
            txt = timestamp + txt
        self.tx.insert('end', txt)
        self.tx.see('end')

    def quit(self):
        self.top.destroy()

if __name__ == '__main__':
    App(open_comport())

