#!/usr/bin/python3
# -*- coding: UTF-8 -*-
#       17.12.2015


import serial
import sys
import tkinter as tk
from tkinter import ttk

comport_name = ""

def get_comport_name():
    return comport_name

def open_comport(comport=None, start=1, end=13, timeout=0.1):
    """
    Prüft, ob sich die Ports COM1: bis COM13: öffnen lassen
    gibt eine Liste der öffenbaren Ports zurück.
    Falls genau 1 Port gefunden wurde, wird dieser geöffnet.

    Werden mehrere gefunden, dann kann eine Vorgabe unter
    "comport" festgelegt werden.

    Oder es wird der Suchbereich offener Ports durch die
    Parameter 'start' und 'end' eingegrenzt:
        open_serport(start = 9, end = 10) findet nur den Port 9

    Rückgabe: serielle Schnittstelle
    """
# Die Ports können als "COM1" .. "COM10" oder
# über eine Ziffer      0     ..  9 angesprochen werden
    global comport_name

    ports = []
    if comport is not None:
        default_comport = comport.upper()
        if len(comport) == 1:

            try:
                i = int(comport)
                default_comport = "COM{}".format(str(i))

            except:
                print("ungültiger Parameter 'comport = {}'".format(comport))
                print("Exit\n")
                exit()

    else:
        default_comport = None

    # versuchen, die Comport im Bereich von 'start' bis 'end' zu öffnen
    # die gefundenen Ports werden in die Liste 'ports' übernommen

    opsys = sys.platform
    if opsys == "win32":
        for i in range(start, end):
            try:
                p = "COM" + str(i)
                rx = serial.Serial(p)
                rx.close()
                ports.append(p)

            except IOError:
                pass

    elif opsys == "linux2":
        # Vielleicht arbeiten wir auf einem RaspberryPi ??
        try:
            p = "/dev/ttyAMA0"
            rx = serial.Serial(p)
            rx.close()
            ports.append(p)

        except IOError:
            pass

    else:
        print("OS =", opsys)
        print("quit")
        exit()

    print("Freie Ports: ", ports)

# Jetzt den richtigen Port öffnen
    if len(ports) == 1:     # genau ein freier Port gefunden
        if default_comport is None or default_comport in ports:
            comport = serial.Serial(ports[0], 9600, rtscts=False)
        else:
            print("Port <{}:> konnte nicht geöffnet werden !".format(default_comport))
            exit()

    elif len(ports) == 0:   # Kein freier Port gefunden
        print("Kein freier Port gefunden !")
        exit()

    else:                   # mehrere freie Ports gefunden
        if default_comport is not None and default_comport in ports:
            comport = serial.Serial(default_comport, 9600)

        else:
            s = SerDialog(ports)
            comport = s.start()

            if comport is not None:
                comport = serial.Serial(comport, 9600)

            else:
                print("Es wurde kein Port ausgewählt !")
                print("Exit\n")
                exit()

    print("geöffnet wurde: {}\n".format(comport.name))
    comport.timeout = timeout
    comport_name = comport.name

    return comport


class SerDialog(object):
    """ Ein Dialog zur Auswahl einer Seriellen Schnittstelle """
    def __init__(self, portlist):
        self.portlist = portlist

        self.top = tk.Tk()
        self.top.title("open_serport()")
        self.top.protocol('WM_DELETE_WINDOW', self._quit)
        self.top.bind('<Escape>', self._quit)
        self.top.resizable(width=False, height=False)   # Größe nicht veränderbar

        self.top.bell()

        self.frm = ttk.LabelFrame(self.top, text="Auswahl Serielle Schnittstelle")
        self.frm.grid(column=0, row=0, padx=5, pady=5, sticky="NSEW")

        self.combo = ttk.Combobox(self.frm, values=self.portlist,
                state="readonly", height=2, width=6)
        self.combo.set(self.portlist[0])
        self.combo.grid(column=0, row=1, padx=5, pady=5, sticky="nswe")
        self.combo.bind("<<ComboboxSelected>>", self._change_selection)

        self.lbl = ttk.Label(self.frm, text="COM-Port auswählen:")
        self.lbl.grid(column=0, row=0, columnspan=2, padx=5, pady=5, sticky="NSEW")

        self.btn1 = ttk.Button(self.frm, text="Übernehmen",  width=14, command=self.übernehmen)
        self.btn1.grid(column=2, row=1, padx=5, pady=5, sticky="NSEW")
        self.btn1.bind('<Return>', self.übernehmen)
        self.btn1.focus_set()

        self.btn2 = ttk.Button(self.frm, text="Beenden", width=14, command=self._quit)
        self.btn2.grid(column=3, row=1, padx=5, pady=5, sticky="NSEW")
        self.btn2.bind('<Return>', self._quit)

    def start(self):
        # Vorgabe für die Auswahl ist das erste Element der Liste
        # dies wird zurückgegeben, wenn keine andere Auswahl getroffen wird
        self.selection = self.portlist[0]
        self.top.mainloop()
        return self.selection

    def _change_selection(self, event=None):
        # Holt die Bezeichnung des in der Liste gewählten Elementes
        j = event.widget.current()
        self.selection = self.portlist[j]

    def übernehmen(self, e=None):
        self.top.destroy()

    def _quit(self, e=None):
        # Bei einem Abbruch wird None zurückgegeben.
        self.selection = None
        self.top.destroy()
