#!/usr/bin/python3
# -*- coding: UTF-8 -*-

# 30.09.2015

import threading, time

START_BYTE = 0xA5


class RS232(threading.Thread):
    """ Empfängt im Hintergrund Daten via Serielle Schnittstelle
        Ein Datensatz beginnt mit dem Startbyte 0xA5 und ist insgesamt 3 Byte lang.
        Byte1 und Byte2 werden an die Callback-Funktion übergeben
    """
    def __init__(self, ser = None, callback = None):
        threading.Thread.__init__(self)

        self.ser =ser
        self.cb = callback
        self.__ende = False


    def quit(self):
        """ Beendet den Hintergrund-Thread """
        self.__ende = True


    def run(self):
        self.ser.flushInput()

        while not self.__ende:
            rx = self.ser.inWaiting()
            # Der Datenstazt besteht aus genau 3 Byte (START_BYTE, Interrupt_Status, Datenbyte)
            # Um den Header zu idenifizieren wird zunächst nur 1 Byte eingelesen
            # Wenn das korrekt ist, dann wird auf die weiteren 3 Bytes gewartet
            if rx:
                v = self.ser.read(1)

                # das erste Byte muss 0xA5 sein
                # wenn nicht, dann liegt kein Datensatzanfang vor
                if not v[0] == START_BYTE:
                    print("Falsches Startbyte: {:4x}".format(v[0]))
                    continue

                while self.ser.inWaiting() < 2:
                    time.sleep(0.001)       # 1ms warten

                v = self.ser.read(2)        # die nächsten 2 Byte einlesen

                if self.cb is not None:
                    self.cb(v)

            time.sleep(0.025)

        if not self.ser.closed:
            self.ser.close()

        print("Thread 'RS232_i' stopped")
        exit()

